@@ -1,6 +1,6 @@
+use std::fmt;
 use std::ops;
 use std::str::FromStr;
-use std::fmt;
 
 use serde::{Deserialize, Serialize};
 
@@ -225,7 +225,7 @@ impl Chromatic {
     pub fn parse_span(from: &str, notation: Notation) -> Option<(Chromatic, usize)> {
         use self::Notation::*;
 
-        if from.len() == 0 {
+        if from.is_empty() {
             None
         } else {
             match notation {
@@ -340,7 +340,6 @@ impl fmt::Display for Chromatic {
     }
 }
 
-
 #[derive(Clone, Debug)]
 pub struct Chord {
     base: Chromatic,
@@ -363,7 +362,7 @@ impl Chord {
     /// bytes) til separator position (if any). Returns `None` if the chord
     /// can't be parsed.
     fn parse_one(from: &str, notation: Notation) -> Option<(Chord, usize)> {
-        if from.len() == 0 {
+        if from.is_empty() {
             return None;
         }
 
@@ -372,6 +371,7 @@ impl Chord {
         let uppercase = first_char.is_uppercase() || first_char.is_numeric();
 
         let mut sep_found = false;
+        let len = from.len();
         let end = from[base_size..]
             .char_indices()
             .find(|(_, c)| {
@@ -383,7 +383,7 @@ impl Chord {
                 false
             })
             .map(|(idx, _)| idx + base_size)
-            .unwrap_or(from.len());
+            .unwrap_or(len);
 
         let chord = Chord {
             base,
@@ -467,7 +467,6 @@ impl fmt::Display for Chord {
     }
 }
 
-
 #[cfg(test)]
 mod tests {
     use super::*;
