@@ -1,15 +1,15 @@
 use std::mem;
 use std::str;
 
+use comrak::nodes::{AstNode, ListType, NodeCode, NodeValue};
 use comrak::{ComrakExtensionOptions, ComrakOptions, ComrakParseOptions, ComrakRenderOptions};
-use comrak::nodes::{AstNode, NodeValue, ListType};
 use lazy_static::lazy_static;
-use regex::{Regex, Captures};
+use regex::{Captures, Regex};
 
 use crate::book::*;
-use crate::util::{BStr, ByteSliceExt};
-use crate::music::{self, Notation};
 use crate::error::*;
+use crate::music::{self, Notation};
+use crate::util::{BStr, ByteSliceExt};
 
 type AstRef<'a> = &'a AstNode<'a>;
 type Arena<'a> = comrak::Arena<AstNode<'a>>;
@@ -79,12 +79,10 @@ impl Extension {
         if let Some(xpose) = self.try_parse_xpose() {
             // Transposition extension recognized
             Some(Inline::Transpose(xpose))
-        } else if let Some(chorus_ref) = self.try_parse_chorus_ref() {
-            // Chorus reference extension recognized
-            Some(Inline::ChorusRef(chorus_ref))
         } else {
-            // No recognized extension, just push as regular text
-            None
+            // Try parsing chorus reference,
+            // push as regular text if not recognized
+            self.try_parse_chorus_ref().map(Inline::ChorusRef)
         }
     }
 }
@@ -241,7 +239,9 @@ impl<'a> NodeExt<'a> for AstNode<'a> {
         fn recurse<'a>(this: &'a AstNode<'a>, res: &mut String) {
             let value = this.data.borrow();
             let text_b = match &value.value {
-                NodeValue::Text(b) | NodeValue::Code(b) => Some(b),
+                NodeValue::Text(literal) | NodeValue::Code(NodeCode { literal, .. }) => {
+                    Some(literal)
+                }
                 _ => None,
             };
 
@@ -323,26 +323,27 @@ impl<'a> NodeExt<'a> for AstNode<'a> {
     }
 }
 
-
 #[derive(Debug)]
 struct ChordBuilder {
     chord: BStr,
     alt_chord: Option<BStr>,
+    backticks: usize,
     inlines: Vec<Inline>,
     line: u32,
 }
 
 impl ChordBuilder {
-    fn new(chord: BStr, line: u32) -> Self {
+    fn new(code: &NodeCode, line: u32) -> Self {
         Self {
-            chord,
+            chord: code.literal.as_bstr(),
             alt_chord: None,
+            backticks: code.num_backticks,
             inlines: vec![],
             line,
         }
     }
 
-    fn inlines_mut<'s>(&'s mut self) -> &'s mut Vec<Inline> {
+    fn inlines_mut(&mut self) -> &mut Vec<Inline> {
         &mut self.inlines
     }
 
@@ -371,7 +372,7 @@ impl ChordBuilder {
     }
 
     fn finalize(self, inlines: &mut Vec<Inline>) {
-        let chord = Chord::new(self.chord, self.alt_chord, self.line);
+        let chord = Chord::new(self.chord, self.alt_chord, self.backticks, self.line);
         let chord = Inline::Chord(Inlines {
             data: chord,
             inlines: self.inlines.into(),
@@ -380,7 +381,6 @@ impl ChordBuilder {
     }
 }
 
-
 #[derive(Debug)]
 struct VerseBuilder<'a> {
     label: VerseLabel,
@@ -400,7 +400,10 @@ impl<'a> VerseBuilder<'a> {
     }
 
     fn with_p_nodes<'n, I>(
-        label: VerseLabel, xp: Transposition, config: &'a ParserConfig, mut nodes: I,
+        label: VerseLabel,
+        xp: Transposition,
+        config: &'a ParserConfig,
+        mut nodes: I,
     ) -> Result<Self>
     where
         I: Iterator<Item = AstRef<'a>>,
@@ -497,14 +500,14 @@ impl<'a> VerseBuilder<'a> {
                 assert!(text.is_text());
                 let text = text.as_plaintext().into();
 
-                let link = Link::new(link.url.into_bstr(), link.title.into_bstr(), text);
+                let link = Link::new(link.url.as_bstr(), link.title.as_bstr(), text);
                 Inline::Link(link)
             }
             NodeValue::Image(link) => {
                 let img = Image::new(
-                    link.url.into_bstr(),
+                    link.url.as_bstr(),
                     node.as_plaintext().into(),
-                    link.title.into_bstr(),
+                    link.title.as_bstr(),
                 );
                 Inline::Image(img)
             }
@@ -532,7 +535,7 @@ impl<'a> VerseBuilder<'a> {
                     cb.finalize(&mut para);
                 }
 
-                let mut new_cb = ChordBuilder::new(code.into_bstr(), c_data.start_line);
+                let mut new_cb = ChordBuilder::new(code, c_data.start_line);
                 if self.xp.is_some() {
                     new_cb.transpose(&self.xp).with_context(|| {
                         format!(
@@ -589,7 +592,7 @@ impl<'a> VerseBuilder<'a> {
     }
 
     fn finalize(self) -> (Verse, Transposition) {
-        let verse = Verse::new(self.label.into(), self.paragraphs);
+        let verse = Verse::new(self.label, self.paragraphs);
         (verse, self.xp)
     }
 }
@@ -603,13 +606,14 @@ struct SongBuilder<'a> {
     blocks: Vec<Block>,
     xp: Transposition,
     config: &'a ParserConfig,
+    verse_num: u32,
 }
 
 impl<'a> SongBuilder<'a> {
     fn new(nodes: &'a [AstRef<'a>], config: &'a ParserConfig) -> Self {
         // Read song title or use fallback
         let (title, nodes) = match nodes.first() {
-            Some(n) if n.is_h(1) => (n.as_plaintext().into(), &nodes[1..]),
+            Some(n) if n.is_h(1) => (n.as_plaintext(), &nodes[1..]),
             _ => (config.fallback_title.clone(), nodes),
         };
 
@@ -631,9 +635,15 @@ impl<'a> SongBuilder<'a> {
             blocks: vec![],
             xp: Transposition::new(config.notation, config.xp_disabled),
             config,
+            verse_num: 0,
         }
     }
 
+    fn next_verse_num(&mut self) -> u32 {
+        self.verse_num += 1;
+        self.verse_num
+    }
+
     fn verse_mut<'s>(&'s mut self) -> &'s mut VerseBuilder<'a> {
         if self.verse.is_none() {
             self.verse = Some(VerseBuilder::new(
@@ -669,7 +679,7 @@ impl<'a> SongBuilder<'a> {
                     prev_bq = false;
                 }
 
-                if !self.verse.is_some() {
+                if self.verse.is_none() {
                     let label = VerseLabel::Chorus(Some(level));
                     let verse = VerseBuilder::new(label, self.xp.clone(), &self.config);
                     self.verse = Some(verse);
@@ -692,11 +702,11 @@ impl<'a> SongBuilder<'a> {
                 NodeValue::Paragraph => self.verse_mut().add_p_node(node)?,
 
                 NodeValue::List(list) if matches!(list.list_type, ListType::Ordered) => {
-                    for (i, item) in node.children().enumerate() {
+                    for item in node.children() {
                         assert!(item.is_item());
                         self.verse_finalize();
 
-                        let label = VerseLabel::Verse((list.start + i) as u32);
+                        let label = VerseLabel::Verse(self.next_verse_num());
                         let verse = VerseBuilder::with_p_nodes(
                             label,
                             self.xp.clone(),
@@ -730,7 +740,7 @@ impl<'a> SongBuilder<'a> {
                 }
 
                 NodeValue::CodeBlock(cb) => self.blocks.push(Block::Pre {
-                    text: cb.literal.into_bstr(),
+                    text: cb.literal.as_bstr(),
                 }),
 
                 _ => {}
@@ -759,7 +769,7 @@ impl<'a> SongBuilder<'a> {
         let mut song = Song {
             title: self.title.into(),
             subtitles: self.subtitles.into(),
-            blocks: self.blocks.into(),
+            blocks: self.blocks,
             notation: self.xp.src_notation,
         };
 
@@ -897,7 +907,7 @@ impl<'i> Parser<'i> {
         //    as of now this is just removing of empty paragraphs/verses,
         //    this is actually implemented on the Song AST type (in book).
         //
-        // The Result is on or more Song structures which are appended to the songs vec passed in.
+        // The Result is one or more Song structures which are appended to the songs vec passed in.
         // See the book module where the bard AST is defined.
 
         let orig_len = songs.len();
@@ -943,6 +953,13 @@ mod tests {
         }
     }
 
+    fn get_verse(song: &Song, block_num: usize) -> &Verse {
+        match &song.blocks[block_num] {
+            Block::Verse(verse) => verse,
+            b => panic!("Unexpected block type: {:?}", b),
+        }
+    }
+
     #[test]
     fn songs_split() {
         let input = r#"
@@ -1159,10 +1176,10 @@ Some lyrics.
 "#;
 
         let song = parse_one(input);
-        assert_eq!(&*song.subtitles, &[
-            "Subtitle 1".into(),
-            "Subtitle 2".into(),
-        ]);
+        assert_eq!(
+            &*song.subtitles,
+            &["Subtitle 1".into(), "Subtitle 2".into(),]
+        );
     }
 
     #[test]
@@ -1170,7 +1187,7 @@ Some lyrics.
         let input = r#"
 # Song
 1. Sailing round `G`the ocean,
-Sailing round the `D`sea.
+Sailing round the ```D```sea.
 "#;
         parse_one_para(input).assert_eq(json!([
             { "type": "i-text", "text": "Sailing round " },
@@ -1178,6 +1195,7 @@ Sailing round the `D`sea.
                 "type": "i-chord",
                 "chord": "G",
                 "alt_chord": null,
+                "backticks": 1,
                 "inlines": [{ "type": "i-text", "text": "the ocean," }],
             },
             { "type": "i-break" },
@@ -1186,6 +1204,7 @@ Sailing round the `D`sea.
                 "type": "i-chord",
                 "chord": "D",
                 "alt_chord": null,
+                "backticks": 3,
                 "inlines": [{ "type": "i-text", "text": "sea." }],
             },
         ]));
@@ -1205,6 +1224,7 @@ Sailing_ round the `D`sea.**
                 "type": "i-chord",
                 "chord": "G",
                 "alt_chord": null,
+                "backticks": 1,
                 "inlines": [{
                     "type": "i-strong",
                     "inlines": [
@@ -1225,6 +1245,7 @@ Sailing_ round the `D`sea.**
                 "type": "i-chord",
                 "chord": "D",
                 "alt_chord": null,
+                "backticks": 1,
                 "inlines": [{
                     "type": "i-strong",
                     "inlines": [{ "type": "i-text", "text": "sea."  }]
@@ -1363,12 +1384,14 @@ Mixed !>> in text.
                     "type": "i-chord",
                     "chord": "Em",
                     "alt_chord": "Hm",
+                    "backticks": 1,
                     "inlines": [ { "type": "i-text", "text": "Yippie yea " } ],
                   },
                   {
                     "type": "i-chord",
                     "chord": "G",
                     "alt_chord": "D",
+                    "backticks": 1,
                     "inlines": [ { "type": "i-text", "text": "oh!" } ],
                   },
                   { "type": "i-break" },
@@ -1377,6 +1400,7 @@ Mixed !>> in text.
                     "type": "i-chord",
                     "chord": "Bm",
                     "alt_chord": "Hm",
+                    "backticks": 1,
                     "inlines": [ { "type": "i-text", "text": "yay!" } ],
                   },
                 ]
@@ -1384,4 +1408,37 @@ Mixed !>> in text.
             }
         ]));
     }
+
+    #[test]
+    fn parse_verse_numbering() {
+        let input = r#"
+# Song 1
+
+1. Verse 1.
+> Chorus 1.
+1. Verse 2.
+>> Chorus 2.
+1. Verse 3.
+
+# Song 2
+
+1. Verse 1.
+2. Verse 2.
+> Chorus.
+>> Chorus two.
+3. Verse 3.
+3. Verse 3.
+"#;
+
+        let songs = parse(input, true);
+
+        assert_eq!(get_verse(&songs[0], 0).label, VerseLabel::Verse(1));
+        assert_eq!(get_verse(&songs[0], 2).label, VerseLabel::Verse(2));
+        assert_eq!(get_verse(&songs[0], 4).label, VerseLabel::Verse(3));
+
+        assert_eq!(get_verse(&songs[1], 0).label, VerseLabel::Verse(1));
+        assert_eq!(get_verse(&songs[1], 1).label, VerseLabel::Verse(2));
+        assert_eq!(get_verse(&songs[1], 4).label, VerseLabel::Verse(3));
+        assert_eq!(get_verse(&songs[1], 5).label, VerseLabel::Verse(4));
+    }
 }
