@@ -1,12 +1,12 @@
-pub mod template;
 pub mod json;
+pub mod template;
 
 use anyhow::Result;
 use serde::Serialize;
 
-use crate::ProgramMeta;
 use crate::book::Song;
 use crate::project::{Metadata, Output, Project};
+use crate::ProgramMeta;
 
 #[derive(Serialize, Debug)]
 pub struct RenderContext<'a> {
@@ -20,5 +20,5 @@ pub trait Render {
     fn render<'a>(project: &'a Project, output: &'a Output) -> Result<&'a Output>;
 }
 
-pub use self::template::{DefaultTemaplate, RHtml, RTex, RHovorka};
 pub use self::json::RJson;
+pub use self::template::{DefaultTemaplate, RHovorka, RHtml, RTex};
